//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl), Kee-Eung Kim (kek)   
//  FILE: PCRuleInvInvokeStmt.C
//  PROJ: COKO
//  DATE: 5/5/97
//
// SYNOPSIS
//  Method definitions for CRuleInvInvokeStmt class.
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCState.H"
#include "PCRuleInvInvokeStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CRuleInvInvokeStmt
// Parameters: (CRule*, char*)
//
// Constructor for inverse rule invokation statement.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRuleInvInvokeStmt::CRuleInvInvokeStmt(CRule *rule, char *iname):
  _first(rule),
  _second(NULL)
{
  _first = rule;
  if (iname)
  	{
  	  _second = new char[strlen(iname)+1];
  	  strcpy(_second, iname);
  	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CRuleInvInvokeStmt
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRuleInvInvokeStmt::~CRuleInvInvokeStmt(void)
{
  _first = NULL;
  if (_second) { delete [] _second; _second = NULL; }

  // We do not delete _first in the destructor. The rule will be deleted
  // outside CStmts explicitly. Don't worry about it!
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters: (CState*)
//
//  Executes the given statement.
//  Takes the state of the program before the statement
//  is executed, and then returns the state of the 
//  program after the statement is executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CRuleInvInvokeStmt::Exec(CState *s)
{

  if(_debug_)
	cout << "-- RuleInvInvokeStmt Begins" << endl;

  NonType *old_root = NULL;

  if (s->Store())
	old_root = s->Store()->Copy_kTree();
  else
	{
	  cout << "Error @ CRuleInvInvokeStmt : Exec" << endl;
	  cout << "_Store is NULL error " << endl;
	  exit(1);
	}

  CIdent *matchNode;
  int is_matchNode_new_one = 0;


  // If we are invoking with an identifier, matchNode will have information
  // about the matching node in the query tree. Else (we are invoking with
  // local root of the query tree), matchnode information will correspond
  // to the local root node.
  if(_second) 
	{
	  s->IDeleteInvDep(_second);
	  matchNode = s->IFindDeep(_second);
	}
  else 
	{
	  s->CDeleteInvDep();
	  s->CCleanInvDep();
	  matchNode = new CIdent(s->Store(), 1);
	  is_matchNode_new_one = 1;
	}

  // Make the local root be the matching node. 
  s->SetStore(matchNode->Child());

  // Flush out the identifier environment 
  s->INew();

  // Inverse-execute!
  s = _first->InvExec(s);

  // Store the success flag in a temporary storage.
  int success = s->ISuccess();

  // Restore the identifier environment.
  s->IOld();

  // s->Store() points to the transformed query. Make the new information
  // available in the identifier environment.
  if(_second) s->IModifyFromHere(_second, 
							 s->Store(), 
							 matchNode->Which(), s->IEnv()->getData());

  // If the matching node was not the overall root of the tree, 
  // return to the old local root of the tree.
  if (!is_matchNode_new_one) s->SetStore(old_root);

  // Build it referring the current pattern in the last match (or match
  // each child) statement. The reconstruction will happen on the subtree
  // of local root of the tree.
  if(_second) 
	{
	  s->SetPatternDeep(_second,NULL);		
	  updateDEP(_second, s, s->IEnv()->getData(), 1);
	  if (s->CurrentPattern())
		s->CurrentPattern()->buildCurrent(s);
	  else
		{
		  cout << "Error @ CRuleInvInvokeStmt : Exec" << endl;
		  cout << "current pattern is NULL error " << endl;
		  exit(1);
		}
	}

  // Set it success value of Rule inverse invoke
  s->SetSuccess(success);

  if(_debug_)
	cout << "-- RuleInvInvokeStmt Ends" << endl;

 if (is_matchNode_new_one) 
	{ 
	  delete old_root; old_root = NULL; 
	  delete matchNode; matchNode = NULL; 
	}

  return s;
}

