//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl), Kee-Eung Kim (kek)
//  FILE: PCIEnv.H
//  PROJ: COKO
//  DATE: 5/5/97
//
// SYNOPSIS
//  Classes for Identifier Search List.
//  
//////////////////////////////////////////////////////////////////////////////
#ifndef _PCIENV_H_
#define _PCIENV_H_

#include "PCIdent.H"
#include "DEPList.H"

class CIdent;

//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: CIEnvValueType
//
// SYNOPSIS
//  _key corresponds to the name of the identifier,
//  and _content stores CIdent type entry matching the _key.
//  
//////////////////////////////////////////////////////////////////////////////

class CPattern;

class CIEnvValueType
{
private:
  char *_key;                          
  // Identifier name string

  CIdent * _content;
  // The content

  DEPList * _DEPList;
  // dependency list

  DEPList * _InvDEPList;
  // inverse dependency list

  CPattern * _pattern;
  // pattern associated with the variable
  // We need this for reconstructing job 
  // after invoking any rule to the variable

public:
  int insertDepList(char * a);
  // Insert an item into the dependency list

  int cleanDepList(void);
  // delete all item from the dependency list

  int emptyDepList(void);
  // is dependency list empty? 

  int findDepList(const char*);
  // is item in the dependency list?

  char* popInvDepList(void);
  // pop an item from inverse dependency list
  // and return it

  int insertInvDepList(char * a);
  // Insert entry into the inverse dependency list

  int cleanInvDepList(void);
  // delete all item from the inverse dependency list

  int emptyInvDepList(void); 
  // is inverse dependency list empty?

  int findInvDepList(const char*);
  // is item in the inverse dependency list?

  const char * GetKey(); 
  // Returns _key

  CIdent * GetContent();           
  // Returns _content

  DEPList * GetList();                    
  // Returns _DEPList

  DEPList * GetInvList();
  // Returns _InvDEPList

  CPattern * GetPattern();
  // Returns _pattern

  void SetKey(char *);
  // _key modifier

  void SetContent(CIdent *);
  // _content modifier

  void SetPattern(CPattern*);          
  // _pattern modifier

  CIEnvValueType(const char *, CIdent *);   
  // Constructor with _key and _content

  ~CIEnvValueType();
  // Destructor
};


//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: CIEnvNodeType
//
// SYNOPSIS
//  CIEnvValueType's will be stored as linked list. CIEnvNodeType
//  defines structure of each node in the stack.
//
//////////////////////////////////////////////////////////////////////////////

class CIEnvNodeType
{
private:
  CIEnvValueType * _data;
  // The real data item 

public:
  CIEnvNodeType *_pre;  

  CIEnvNodeType *_next;
     
  
  CIEnvValueType * GetData();
  // Returns _data

  void SetData(CIdent*);
  // _data modifier

  CIEnvNodeType(void);
  // Parameterless constructor

  CIEnvNodeType(CIEnvValueType * data);
  // Constructor for a node with data

  ~CIEnvNodeType();
  // Destructor

  int insertDepList(char* a);
  // Insert an item into the dependency list of _data.

  int emptyDepList(void);
  // Is dependency list of _data empty? 
  
  int findDepList(const char*);
  // Is item in the dependency list of _data?

  char* popInvDepList();
  // Pop an item from inverse dependency list of _data
  // and return it.

  int insertInvDepList(char* a);
  // Insert entry into the inverse dependency list of _data.

  int emptyInvDepList(void);
  // Is inverse dependency list of _data empty?

  int findInvDepList(const char*);
  // Is item in the inverse dependency list of _data?

  CPattern* GetPattern(void);
  // Get _pattern of _data.

  void SetPattern(CPattern*);
  // Modify _pattern of _data.

  void initIteratorDEP();
  // Call initIterator method of _DEPList of _data.

  char* getValueDEP();
  // Call getValueDEP method of _DEPList of _data.

};

//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: CIEnv
//
// SYNOPSIS
//  The wrapping class for the identifier search list.
//
//////////////////////////////////////////////////////////////////////////////

class CIEnv 
{
private:
  CIEnvNodeType *_head; 
  CIEnvNodeType *_tail; 

public:
  CIEnvNodeType *_findNode(const char *key); 
  // Internal search function for modify function

  CIdent * find(const char *key);  
  // Search function for a given key

  CIEnv();                    
  // Parameterless constructor

  ~CIEnv(); 
  // Destructor

  CIEnvValueType * first(void); 
  // Returns the top node value

  void insert(CIEnvValueType * data);
  // Inserts a given node value into the stack

  void erase(const char * key);              
  // Find a node with a given key then delete it from the list.

  void modify(char*, CIdent*); 
  // Modifies a given node value in the stack.

};

#endif
