/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

/*
//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)					 	    
//  FILE: KGlobals.H 
//  PROJ: KOLA Parser and The "Karbonator" (AQUA -> KOLA translator)
//  DATE: 5/95		                                                    
//
// SYNOPSIS
//  Global Declarations for Use in KOLA parser and AQUA -> KOLA translator
//
//////////////////////////////////////////////////////////////////////////////
*/

#ifndef KOLA_GLOBALS_HEADER
#define KOLA_GLOBALS_HEADER

#include <stdio.h>
#include <iostream.h> 
#include <string.h>

#ifndef NULL
  #define NULL 0
#endif

#define MAXINTLENGTH 32

static const int F = 0;
static const int T = 1;

#define SAND "/\\"
#define SOR "\\/"
#define IMPLIES "=>"
#define IN "IN"

typedef enum {FNAME, PNAME, ONAME, BNAME} NameTag;

// typedef enum {INDEX_INVALID, INDEX_USER, INDEX_VAR} IndexName;

typedef enum {FUN_INVALID, FUN_CHOOSE, FUN_USER, FUN_SHIFTL, FUN_SHIFTR,
	      FUN_ID, FUN_1, FUN_2, FUN_FLATTEN, FUN_SET,
	      FUN_CELT, FUN_INT, FUN_DIF, FUN_MOD, FUN_ABS,
	      FUN_INVERSE, FUN_UNI, FUN_COMPOSE, FUN_TIMES, FUN_CONST, FUN_NJOIN,
	      FUN_ADD, FUN_MULT, FUN_DIV, FUN_MINUS,
	      FUN_PLUS, FUN_UNNEST, FUN_JOIN, FUN_PCON, 
	      FUN_PFOLD, FUN_CON, FUN_FOLD, FUN_SINGLETON,
          FUN_INS, FUN_SORT, FUN_RSJOIN, FUN_LSJOIN,
          FUN_ITERATE, FUN_ITER, FUN_VAR, FUN_DC, FUN_AVG, FUN_SUM, FUN_COUNT, FUN_MIN, FUN_MAX, FUN_TID} FuncName;

typedef enum {BOOL_AND, BOOL_NEQ, BOOL_EQU, BOOL_GE, BOOL_GT, BOOL_INVOKE,
              BOOL_LE, BOOL_LT, BOOL_NAME, BOOL_NOT, BOOL_OR, 
              BOOL_PRIM, BOOL_VAR, BOOL_DC} BoolName;

typedef enum {OBJ_ANY, OBJ_MOD, OBJ_DIF, OBJ_DIV, OBJ_IF, OBJ_INT, OBJ_INTER, OBJ_FLOAT,
              OBJ_INVOKE, OBJ_MINUS, OBJ_MULT, OBJ_NAME, OBJ_PAIR, OBJ_PLUS, 
              OBJ_SET, OBJ_STR, OBJ_TIMES, OBJ_UNI, OBJ_VAR, OBJ_DC} ObjName;

typedef enum {PRED_INVALID, PRED_EQUAL, PRED_USER, PRED_LESS, PRED_INV,
	      PRED_NEGATE, PRED_FORALL, PRED_EXISTS, PRED_CELT, PRED_DISJUNCT, PRED_CONJUNCT, 
	      PRED_TIMES, PRED_CONST, PRED_GREATER, PRED_LESSEQ,
	      PRED_GREATEREQ, PRED_NOTEQ, PRED_COMBINE, PRED_CEX, PRED_CFA, 
	      PRED_VAR, PRED_DC, PRED_LEX, PRED_ISNULL, PRED_ISNOTNULL} PredName;

typedef enum {TYPE_BOOL, TYPE_INT, TYPE_PAIR, TYPE_BAG, TYPE_FLT, TYPE_SET, TYPE_STR, TYPE_TUP, TYPE_CHAR,
              TYPE_UNK, TYPE_USER, TYPE_DC, TYPE_VAR, TYPE_ERROR, TYPE_ANY, TYPE_THOR} TypeName;

class FTID;
class FSort;
class FShiftL;
class FShiftR;
class FIns;
class FRSJoin;
class FLSJoin;
class FPCon;
class koObject;
class Function;
class Environment;
class FMod;
class FAbs;
class FInverse;
class TFunc;
class TObj;
class TError;
class Predicate;
class TPred;
class Type;
class NonType;
class FIterate;
class FIter;
class FSingleton;
class F1;
class FAdd;
class FMinus;
class FDiv;
class FMult;
class FSet;
class FUser;
class FCompose;
class FChoose;
class F2;
class FTimes;
class FRJoin;
class FID;
class FUnnest;
class FFold;
class FPFold;
class FPlus;
class FJoin;
class FConst;
class FInt;
class FUni;
class FNJoin;
class FCon;
class FCElt;
class FDif;
class PEqual;
class PGreater;
class PGreaterEqual;
class PLessEqual;
class PNotEqual;
class FFlatten;
class PLess;
class PTimes;
class PInv;
class PDisjunct;
class PForall;
class PExists;
class PConst;
class PCElt;
class PConjunct;
class PNegate;
class PSubset;
class PCombine;
class BNot;
class OIf;
class OPair;
class BInvoke;
class OName;
class BName;
class OInt;
class OAny;
class TVar;
class BPrimitive;
class TPair;
class TSet;
class TBag;
class TFlt;
class TThor;
class TTup;
class TUnk;
class TAny;
class koBool;
class PUser;
class BAnd;
class BOr;
class TStr;
class TChar;
class TBool;
class ObjList;
class TInt;
class TUser;
class Schema;
class SchemaClassEntry;
class SchemaSignature;
class OSet;
class OStr;
class OFloat;
class ODif;
class OTimes;
class OInter;
class OUni;
class OInvoke;
class OLink;
class OPlus;
class OMult;
class OMod;
class ODiv;
class OMinus;
class OId;
class BEqu;
class BNotEqual;
class BLT;
class BGE;
class BLE;
class BGT;
class PCEX;
class PCFA;
class PIsnull;
class PIsnotnull;
class NameUnion;
#endif

