/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
// FILE: KOFCount.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 7/98
//
//////////////////////////////////////////////////////////////////////////////

#include "KOFCount.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FCount
// Parameters:	(TFunc*)							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FCount::FCount (TFunc * ftype)  : Function (ftype, FUN_COUNT, "COUNT")
{    
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FCount
// Parameters:	(FCount& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FCount::FCount (const FCount& other) : Function ((Function &) other)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~FCount
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FCount::~FCount ()
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as Function*
//         								    
//////////////////////////////////////////////////////////////////////////////

Function* FCount::Copy () const
{
  return new FCount (*this);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  deepeq
// Parameters:	()							  
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//         								    
//////////////////////////////////////////////////////////////////////////////

int FCount::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == FNAME ) && (name_->fn == FUN_COUNT))
  {
    delete name_;
    return T;
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* FCount::Myname()
{
	NameUnion* a = new NameUnion(FNAME, FUN_COUNT);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  return 
//    ((OResPAIR *) a) -> fst -> reduce ()
//
/////////////////////////////////////////////////////////////////////////////

// OResult* FCount::reduce (OResult* a)
// {
//   if (a -> isPair ()) {
//     OResPAIR *a2 = ((OResPAIR *) a);
//     OResINF* res = new OResINF (a2 -> One () -> Copy (), "+", a2 -> Two () -> Copy ());
//     delete a;
//     return res;
//   } else {
//     OResBB *res = new OResBB ("COUNT", a -> Copy ());
//     delete a;
//     return res;
//   }
// }

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "FPRIM: _Count"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* FCount::representation ()
{
  char *res = new char [strlen ("COUNT") + 1];
  sprintf (res, "%s", "COUNT");
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* FCount::transform(CState* s)
{
    return new FCount(Type());
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\ 
//                    |_\         / building part
//                      / matching part 
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair 
// into the env stack. Then the next time we face another g (second one), 
// we search the stack for the matching pair with variable name g and simply check 
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID). 
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int FCount::compares(NonType* nt, CState* a)
{
  NameUnion* tmp_name = nt->Myname();
  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_COUNT))
	{
	  delete tmp_name;
	  return T;
	}
  else
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void FCount::replaceChild(NonType* child, int which_child)
{
}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: invoke
//  Parameters:(koObject*)
//
//  Counts the two integers in the pair.
//
//////////////////////////////////////////////////////////////////////////////

koObject* FCount::invoke(koObject* x)
{
  return new OInvoke(this,x);
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSObject*)
//
//  Counts the two integers in the pair.
//
//////////////////////////////////////////////////////////////////////////////

OSObject* FCount::exec(OSObject* x)
{
  cerr << "Calling undefined exec on FCount" << endl;
  return NULL;
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* FCount::prolog ()
{
  char *res = new char [strlen ("fcount") + 1];
  sprintf (res, "%s", "fcount");
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref FCount::sp_prolog ()
{
  SP_term_ref res = SP_new_term_ref();
  SP_put_string(res,"fcount");
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FCount::sql (koObject *o)
{    
  char *arg = o -> sql ();
  if (strncmp (arg, "ERROR", 5) == 0)
	return arg;
  else {
	char *res = new char [strlen ("COUNT ()") + strlen (arg) + 1];
	sprintf (res, "COUNT (%s)", arg);
	delete [] arg;
	return res;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FCount::oql (koObject *o)
{    
  char *arg = o -> oql ();
  if (strncmp (arg, "ERROR", 5) == 0)
	return arg;
  else {
	char *res = new char [strlen ("COUNT ()") + strlen (arg) + 1];
	sprintf (res, "COUNT (%s)", arg);
	delete [] arg;
	return res;
  }
}
