/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)					 	    
//  FILE: KOFID.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97	                                                    
//
// SYNOPSIS
//  Identity function object.
//  
//     Id ! e = e
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_FID_HEADER
#define KOLA_FID_HEADER 

#include "KGlobals.H"
#include "KOFunction.H"
#include "KONameUnion.H"
#include "KOOInvoke.H"

#include "PCState.H"

#include "KONonType.H"
#include <string.h>

class FID : public Function
{
public:
  FID (TFunc * ftype = NULL);
  // Constructor

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack.
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.
     
  FID (const FID& other);
  // Copy Constructor
    
  ~FID ();
  // Destructor
    
  Function* Copy () const;
  // Virtual Copy Constructor;
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 0; }
  //  Number of arguments.

  //  OResult* reduce (OResult* a);
  //  Return a
    
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  "FPRIM: ID"

  void initCurrChild() {}
  // This is a dummy function for all the classes 
  // that don't have any child.

  NonType* nextChild() { return NULL; }
  // This is dummy functions for all the classes 
  // that don't have any child.
  // This dummy function returns NULL since there is no next child for 
  // those classes (classes w/o any child.) 

  void setDEPnInvDEP(char*, CState*) {}
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*) {}
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  const char * Name() const { return namestring; }
  // Return the text representation of the name.

  koObject *inv (koObject *o);
  // Symbolic invocation of this on o.  Eventually,
  // this method will replace "invoke" presently
  // only defined with _OSTORE_ flag set

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject* invoke(koObject* x) { return new OInvoke(this, x); }
  //  Returns itself.

  OSObject* exec(OSObject* x) { return x->copy(); }
  //  Returns itself.
#endif // _OSTORE_

 protected:
  
};


#endif





