/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)	
//        Joon S. Lee (jl)				 	    
//  FILE: KOFJoin.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97	                                                    
//
// SYNOPSIS
//  Join function former.  Given a predicate
//  p and a function f, we have:
//  
//    join (p, f) ! [A, B] =
//      { f ! [a, b] | a IN A /\ b IN B /\ p ? [a, b] }
//      
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_FJOIN_HEADER
#define KOLA_FJOIN_HEADER 

#include "KGlobals.H"
#include "KOPredicate.H"
#include "KOFunction.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOOInvoke.H"
#include "KOBPrimitive.H"
#include "KOOPair.H"
#include "KOPConst.H"

#ifdef _OSTORE_
#include "SODA_GLOBALS.H"
#include "JoinSieve.H"
#include "FilterJoinSieve.H"
#include "MapJoinSieve.H"
#include <dlfcn.h>
#endif // _OSTORE_

#include "PCState.H"
#include "KOPVar.H"
#include "KOFVar.H"
#include <string.h>

typedef NonType* (*callOptPtr)(NonType*);

class FJoin : public Function
{
public:
  FJoin (Predicate* p, Function* f, TFunc * ftype = NULL);
  //  Constructor.  p is predicate input and f is function
  //  input.

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.
 
  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  FJoin (const FJoin& other);
  //  Copy Constructor. 
  
  ~FJoin ();
  //  Destructor. 

  Function *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 2; }
  //  Number of arguments.

  Predicate* One () const {return one;}
  //  What is the predicate that is the first argument?
  
  Function* Two () const {return two;}
  //  What is the function that is the second argument?
  
  //  OResult* reduce (OResult* a);
  //  v1 = new OResVAR ();
  //  v2 = new OResVAR ();
  //  
  //  return
  //    new OResSET
  //      (two -> reduce (new OResPAIR
  //        (v1, v2),
  //       new BResCOM
  //         (one -> reduce (new OResPAIR (v1, v2)),
  //          "/\"
  //          new BResCOM
  //             (new BResINF (v1, "IN", a -> fst -> reduce ()),
  //  	    "/\"
  //              new BResINF (v2, "IN", a -> snd -> reduce ()..)
	        
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  "FFO: JOIN (" + one -> representation () + 
  //  		  two -> representation () + ")"

  const char * Name() const { return namestring; }
  // Return the text representation of the name.

  void SetOne(NonType* a) { delete one; one = (Predicate*)a; }
  // replace existing child to new one.
 
  void SetTwo(NonType* a) { delete two; two = (Function*)a; }
  // replace existing child to new one.

  void initCurrChild() {currChild = 0;}
  // set (reset) currChild to 0.
  // This needs to be called before using nextChild() for first time.

  NonType* nextChild();
  // returns next child. (next Child = current Child + 1) 
  //If current child is the last child return NULL.

  koObject *inv (koObject *o);
  // Symbolic invocation of this on o.  Eventually,
  // this method will replace "invoke" presently
  // only defined with _OSTORE_ flag set

  char *oql (koObject *o);
  // Return the OQL string equivalent of invoking this on o

  char *sql (koObject *o);
  // Return the SQL string equivalent of invoking this on o

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject* invoke(koObject* x);

  OSObject* exec(OSObject* x);
  //  Returns a JoinSieve.
#endif // _OSTORE_

protected:
  Predicate* one;
  //  What is the Predicate that is the 1st argument?
  
  Function* two;
  //  What is the function that is the 2nd argument to
  //  the former?
  
  int currChild;
  
};


# endif
