/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)		
//        Joon S. Lee (jl)			 	    
//  FILE: KOFunction.H 
//  PROJ: KOLA
//  DATE: 5/97		                                                    
//
// SYNOPSIS
//  The type of function objects which can
//  be constructed in complex ways and which
//  can be invoked.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_FUNCTION_HEADER
#define KOLA_FUNCTION_HEADER 

#include "KGlobals.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "TYTFunc.H"

#ifdef _OSTORE_
#include "OSObject.H"
#endif // _OSTORE_

#include "PCState.H"

class Function : public NonType
{
public:
  Function ();
  // Constructor.
    
  Function (TFunc *t = NULL, FuncName fn = FUN_INVALID, char * nm = NULL);
  // Constructor.
    
  Function (const Function& other);
  // Copy Constructor.

  virtual ~Function ();
  // Destructor

  virtual Function* Copy () const = 0;
  // Virtual Copy Constructor
    
  NonType* Copy_kTree() const { return Copy(); }
  
  TFunc* Type () const {return this->type;}
  // Selector.    

  FuncName Identity () const {return myname;}
  //  What kind of function are you?
  //  Return myname.
  
  //  virtual OResult* reduce (OResult* a) = 0;
  //  Return a string representation of the semantics
  //  of the function when invoked on expression a.

  virtual char *oql (koObject *o);
  // Return the OQL string equivalent of invoking this on o

  virtual char *sql (koObject *o);
  // Return the SQL string equivalent of invoking this on o

  virtual koObject *inv (koObject *o);
  // Symbolic invocation of this on o.  Eventually,
  // this method will replace "invoke" presently
  // only defined with _OSTORE_ flag set

#ifdef _OSTORE_
  virtual koObject* invoke(koObject* x);
  //  Invokes the symbolic evaluator to transform the
  //  query to a more efficient expression.

  virtual OSObject* exec(OSObject* x) = 0;
  //  Executes the operator on x.
#endif // _OSTORE_

protected:
  TFunc* type;
  //  The type of a function object is an "arrow type".
  
  FuncName myname;
  //  What kind of function are you?

  char * namestring;
  // string representation of the function name.
};

#endif
