/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)					 	    
//  FILE: KOOInt.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97		                                                    
//
// SYNOPSIS
//  Integer objects.  val is the value of the object.
//  According to the language spec, 0 is the only
//  primitive integer and all other integers are formed
//  via the pred and succ formers.  We allow the "shortcut"
//  of decimal notation here.
//    
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_OINT_HEADER
#define KOLA_OINT_HEADER 

#include "KGlobals.H"
#include "KOObject.H"
#include "KONameUnion.H"
#include "KONonType.H"

#ifdef _OSTORE_
#include "OSInt.H"
#endif // _OSTORE_

#include "PCState.H"
#include <string.h>

class OInt : public koObject
{
public:
  OInt (char *val, TObj * t = NULL);
  // Constructor.

  OInt (int val, TObj * t = NULL);
  // Constructor.  

  int compares(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 
 
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  void setDEPnInvDEP(char*, CState*) {}
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*) {}
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  OInt (const OInt& other);
  // Copy Constructor.
  
  ~OInt ();
  // Destructor.

  koObject* Copy () const;
  // Virtual copy constructor.
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 0; }
  //  Number of arguments.

  int Val () const {return val;}
  // Selector
    
  //  OResult* reduce ();
  //  Return as_string (val)
	  
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  Return "OPRIM: INT " + val as a string

  void initCurrChild() {}
  // This is a dummy function for all the classes 
  // that don't have any child.

  NonType* nextChild() { return NULL; }
  // This is dummy functions for all the classes 
  // that don't have any child.
  // This dummy function returns NULL since there is no next child for 
  // those classes (classes w/o any child.) 
  
  const char * Name() const { return namestring; }
  // Returns the text representation of this integer.

  char *sql ();
  // Return the SQL string equivalent of this object

  char *oql ();
  // Return the OQL string equivalent of this object

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject *object() { return this; }
  // Returns the kola equivalent of an object.

  OSObject *result() { return new OSInt(Val()); }
  // Returns the OSObject equivalent of itself.

  int equal(koObject *o) { return (Val() == ((OInt*)o)->Val()); }
  // Checks whether two OInts are equal.

  int unequal(koObject *o) { return (Val() != ((OInt*)o)->Val()); }
  // Checks whether two OInts are not equal.

  int greater(koObject *o) { return (Val() > ((OInt*)o)->Val()); }
  // Checks whether this OInt is > o

  int greaterequal(koObject *o) { return (Val() >= ((OInt*)o)->Val()); }
  // Checks whether this OInt is >= o

  int less(koObject *o) { return (Val() < ((OInt*)o)->Val()); }
  // Checks whether this OInt < o

  int lessequal(koObject *o) { return (Val() <= ((OInt*)o)->Val()); }
  // Checks whether this OInt <= o
#endif // _OSTORE_

protected:
  int val;
  //  What is the value of this integer?

  // #ifdef _OSTORE_
  // OSObject *ostore_obj;
  // pointer to equivalent object stored in ObjectStore
  // #endif // _OSTORE_

};


#endif
