/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)					 	    
//  FILE: KOObject.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97		                                                    
//
// SYNOPSIS
//  The class of containable objects.  (Includes
//  integers, strings, as well as pairs and sets of 
//  Objects).  Also should include user defined 
//  typed objects.  This class and its subclasses
//  construct object that are used in KOLA trees.
//  These should be distinguished from
//  OResults which are used in expression trees.
//  KOLA trees are used for optimization.  Expression
//  trees are used to describe the result of a query denoted 
//  by a KOLA tree.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_OBJECT_HEADER
#define KOLA_OBJECT_HEADER 

#include "KGlobals.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "TYTObj.H"

#ifdef _OSTORE_
#include "OSObject.H"
#endif // _OSTORE_

#include "PCState.H"
#include <iostream.h>

class koObject : public NonType
{
public:
  koObject (TObj *t = NULL, char * nm = NULL);

  koObject (int n, TObj *t = NULL );

  koObject (const koObject & other);

  virtual ~koObject ();

  virtual koObject* Copy () const = 0;
  // Virtual Copy Constructor.

  NonType* Copy_kTree() const { return Copy(); }

  virtual OSet* asSet ()
  {
    cout << "Attempting to convert a non-set object to a set" << endl;
    return NULL;
  }    
  // Coercer.

  virtual OPair* asPair() { return NULL; }

  TObj* Type () const { return this->type; }
  // Selector.
        
  //  virtual OResult* reduce () = 0;
  //  Return the "meaning" of an object, expressed
  //  by reducing the query and using set comprehension
  //  notation to describe sets.

  virtual char *oql ();
  // Return the OQL string equivalent of this object

  virtual char *sql ();
  // Return the SQL string equivalent of this object

  virtual koObject *obj ();
  // Symbolic simplification of this.  Eventually,
  // this method will replace "object" presently
  // only defined with _OSTORE_ flag set

#ifdef _OSTORE_

  virtual koObject *object() = 0;
  // Returns the kola equivalent of an object.

  virtual OSObject *result() = 0;
  // Returns the OSObject equivalent of itself.

  virtual int equal(koObject *o) { return (this == o); }

  virtual int unequal(koObject *o) { return (this != o); }

  virtual int greater(koObject *o) { return (this > o); }

  virtual int greaterequal(koObject *o) { return (this >= o); }

  virtual int less(koObject *o) { return (this < o); }

  virtual int lessequal(koObject *o) { return (this <= o); }
#endif // _OSTORE_

protected:
  TObj* type;
  //  Objects always have some sort of Object Type.
 
  /*********************************************************/
  // right now "myname" is not necessary. Myname() methods does not
  // use this data field. Myname() methods directly manipulates
  // BoolName for each classes.
  // 
  //  ObjName myname;
  //  What kind of Object are you?
  //*******************************************************/

  char * namestring;
  // Text representation for the name of the object.

};
#endif



