/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)					 	    
//  FILE: TYTError.H 
//  PROJ: KOLA
//  DATE: 5/95		                                                    
//
// SYNOPSIS
//  Error Type.  Passed up tree in case of type error.
//    
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_TERR_HEADER
#define KOLA_TERR_HEADER 

#include "KGlobals.H"
#include "TYTObj.H"
#include "ENEnvironment.H"

#include <string.h>

class TError : public TObj
{
    
 public:
  TError (char* n);
  //  Constructor.  n is the error message.

  TError (char* n, TObj *t1);
  //  Constructor.  n is the error message.

  TError (char* n, TObj *t1, TObj *t2);
  //  Constructor.  n is the error message.

  TError (TError & other);
  //  Copy Constructor.  

  ~TError ();
  //  Destructor.

  TObj* Copy ();
    // Virtual Copy Constructor.
    
  char *Msg () {return msg;}
  // Selector.
    
  char* representation ();
  //  return ERROR + msg

  char* prolog ();
  // Return string representation of type in prolog term.

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
  // convert kola tree into prolog term and return it.
#endif // _PROLOG_

  koType* transformation () {return Copy();}      

  int sameas (TObj *other) {return (other -> Identity () == TYPE_ERROR && strcmp (msg, other -> asError () -> Msg ()) == 0);}
  // Error type is same as other error type if error messages are the same.

  TError *asError () {return this;}
  // Coercer
    
 protected:
  char* msg;
  //  What is the error message?
  
};

#endif

