/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)					 	 
// FILE: TYTFunc.C
// PROJ: KOLA Parser, AQUA Translator
// DATE: 5/95
//
//////////////////////////////////////////////////////////////////////////////

#include "TYTFunc.H"


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TFunc
// Parameters:	(TObj* o1,TObj* o2)							  
//
//  Constructor.  Takes domain and range types
//  as arguments.
//         								    
//////////////////////////////////////////////////////////////////////////////

TFunc::TFunc (TObj* o1,TObj* o2)
{
    
    one = o1;
    two = o2;    

}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TFunc
// Parameters:	(TFunc& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

TFunc::TFunc (TFunc& other)
{
    
    one = other . Dom () -> Copy ();
    two = other . Ran () -> Copy ();    

}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~TFunc
// Parameters:	()							  
//
// Destructor.
//        								    
//////////////////////////////////////////////////////////////////////////////

TFunc::~TFunc ()
{

    delete one;
    delete two;  

}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  Return string representation of type.
//  
//    one -> representation () + "-->" + two -> representation ()
//         								    
//////////////////////////////////////////////////////////////////////////////

char* TFunc::representation ()
{
    char *or = one -> representation ();
    char *tr = two -> representation ();
    
    char *res = new char [12 + strlen (or) + strlen (tr)];
    sprintf (res, "FUNC: %s --> %s", or, tr);

	delete [] or;
	delete [] tr;
    
    return res;

}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  prolog
// Parameters:	()							  
//
//////////////////////////////////////////////////////////////////////////////

char* TFunc::prolog ()
{
  char *or, *tr;
  if (one == NULL) {
	or = new char [strlen ("knull") + 1];
	sprintf (or, "knull");
  } else
    or = one -> prolog ();

  if (two == NULL) {
	tr = new char [strlen ("knull") + 1];
	sprintf (tr, "knull");
  } else
    tr = two -> prolog ();

  char *res = new char [strlen (or) + strlen (tr) + strlen ("tfunc(,)") + 1];
  sprintf (res, "tfunc(%s,%s)", or, tr);
  delete [] or;
  delete [] tr;
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref TFunc::sp_prolog ()
{
  SP_term_ref or, tr;
  if (one == NULL) {
	cout << "Calling sp_prolog on TFunc with child #1 = NULL: could be a problem" << endl;
	or = NULL;
  } else
	or = one -> sp_prolog ();

  if (two == NULL) {
	cout << "Calling sp_prolog on TFunc with child #2 = NULL: could be a problem" << endl;
	tr = NULL;
  } else
	tr = one -> sp_prolog ();

  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("tfunc"),2,or,tr);
  return res;
}
#endif // _PROLOG_


TObj* TFunc::Copy ()
{

    return (TObj*) new TFunc (*this);
    
}
